// Project 11: IoT NeoPixel

#include <ESP8266WiFi.h>
#include <ThingerWifi.h>
#include <Adafruit_NeoPixel.h>

#define PIN 4 // GPIO4 = D2
#define PIXELS 7 // 7 pixels op de NeoPixel jewel
#define HELDERHEID 255 // Waarde van 0 tot 255

Adafruit_NeoPixel pixels = Adafruit_NeoPixel(PIXELS, PIN, NEO_GRB + NEO_KHZ800);

ThingerWifi thing("gebruikersnaam", "DEVICE", "CREDENTIAL");

const char* ssid = "SSID";
const char* password = "WACHTWOORD";

int pixel = 0;

uint8_t rood(uint32_t kleur) {
  return kleur >> 16;
}

uint8_t groen(uint32_t kleur) {
  return (kleur >> 8) % 256;
}

uint8_t blauw(uint32_t kleur) {
  return kleur % 256;
}

void setup() {
  Serial.begin(115200);

  // Verbind met WiFi-netwerk
  Serial.println();
  Serial.println();
  Serial.print("Verbind met ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi verbonden");
  Serial.print("IP-adres: ");
  Serial.println(WiFi.localIP());

  pixels.begin();
  pixels.setBrightness(HELDERHEID);
  pixels.show();

  thing["pixel"] << inputValue(pixel);

  thing["rood"] << [](pson& waarde){
    uint32_t kleur = pixels.getPixelColor(pixel);
    pixels.setPixelColor(pixel, pixels.Color(waarde, groen(kleur), blauw(kleur)));
    pixels.show();    
  };

  thing["groen"] << [](pson& waarde){
    uint32_t kleur = pixels.getPixelColor(pixel);
    pixels.setPixelColor(pixel, pixels.Color(rood(kleur), waarde, blauw(kleur)));
    pixels.show();
  };

  thing["blauw"] << [](pson& waarde){
    uint32_t kleur = pixels.getPixelColor(pixel);
    pixels.setPixelColor(pixel, pixels.Color(rood(kleur), groen(kleur), waarde));
    pixels.show();
  }; 

}

void loop() {
    thing.handle();
}
