// Project 12: Toegangscontrole

#include <ESP8266WiFi.h>
#include "MFRC522.h"

#define SDA_PIN 4  // SDA-PIN van mfrc522 aangesloten op GPIO4 (D2) van esp8266
#define RST_PIN 5 // RST-pin van mfrc522 aangesloten op GPIO5 (D1) van esp8266
#define GROEN_PIN 0 // groene led aangesloten op GPIO0 (D3) van esp8266
#define ROOD_PIN 2 // rode led aangesloten op GPIO2 (D4) van esp8266

MFRC522 mfrc522(SDA_PIN, RST_PIN);

const char* ssid = "SSID";
const char* password = "WACHTWOORD";

const char* domoticz_server = "192.168.0.101"; // Wijzig naar het ip-adres van je Domoticz-server.
const int domoticz_poort = 8080;

String getUID() {
  byte buffergrootte = mfrc522.uid.size;
  String uid= "";
  for (byte i = 0; i < buffergrootte; i++) {
    if(mfrc522.uid.uidByte[i] < 0x10)
      uid += "0";
    uid += String(mfrc522.uid.uidByte[i], HEX);
  }
  return uid;
}

String getURL(String uid) {
  if(uid.equalsIgnoreCase("25f5c815"))
    return "/json.htm?type=command&param=switchlight&idx=226&switchcmd=On";
  else if(uid.equalsIgnoreCase("ea661185"))
    return "/json.htm?type=command&param=switchlight&idx=227&switchcmd=On";
  else
    return "";
}

void setup() {
  Serial.begin(115200);

  // Initialiseer leds
  pinMode(GROEN_PIN, OUTPUT);
  pinMode(ROOD_PIN, OUTPUT);
  digitalWrite(GROEN_PIN, LOW);
  digitalWrite(ROOD_PIN, LOW);

  // Initialiseer rfid-module
  SPI.begin();
  mfrc522.PCD_Init();
  
  // Verbind met WiFi-netwerk
  Serial.println();
  Serial.println();
  Serial.print("Verbind met ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi verbonden");
  Serial.print("IP-adres: ");
  Serial.println(WiFi.localIP());

}

void loop() { 

  // Kijk of er zich een nieuwe rfid-kaart bij de lezer bevindt
  if(!mfrc522.PICC_IsNewCardPresent()) {
    digitalWrite(GROEN_PIN, LOW);
    digitalWrite(ROOD_PIN, LOW);
    delay(50);
    return;
  }
  // Lees de kaart
  if(!mfrc522.PICC_ReadCardSerial()) {
    digitalWrite(GROEN_PIN, HIGH);
    digitalWrite(ROOD_PIN, HIGH);
    delay(50);
    return;
  }

  String uid = getUID();
  String url = getURL(uid);

  Serial.print("ID kaart:");
  Serial.println(uid);
  Serial.print("URL:");
  Serial.println(url);
  if(url.equals(""))
    digitalWrite(ROOD_PIN, HIGH);
  else {
    digitalWrite(GROEN_PIN, HIGH);

    Serial.print("Verbind met ");
    Serial.println(domoticz_server);

    WiFiClient client;
    if (!client.connect(domoticz_server, domoticz_poort)) {
      Serial.println("Verbinding mislukt");
      client.stop();
      return;
    }

    Serial.println("Vraag URL op");

    client.print("GET ");
    client.print(url);
    client.println(" HTTP/1.1");
    client.print("Host: ");
    client.print(domoticz_server);
    client.print(":");
    client.println(domoticz_poort);
    client.println("User-Agent: ESP8266");
    client.println("Connection: close");
    client.println();

    client.stop();

  }
}
