// Project 13: Zonne-energie 
#include <ESP8266WiFi.h>

extern "C" {
  #include "user_interface.h"
}

#define TELLER_MEM 64
#define MEETWAARDE_MEM 68
#define MAGIC 0xffffff
#define SECONDE 1000000
#define MINUUT 60000000
#define UUR  3600000000

const char* ssid = "SSID";
const char* password = "WACHTWOORD";
const int puls = 5; // D1
const int sensor = 0; // A0
const int samples = 30; // Aantal samples per meting
const int slapen = UUR; // Tijd tussen twee metingen (in microseconden)

const char* domoticz_server = "192.168.0.101"; // Wijzig naar het ip-adres van je Domoticz-server.
const int domoticz_poort = 8080;
const int idx = 225; // idx van de virtuele sensor in Domoticz, te vinden in Instellingen / Apparaten.

const int weerstand_in_serie = 10000; // 10 kilo-ohm
const int nominale_weerstand = 10000; // nominale weerstand van de ntc
const int nominale_temperatuur = 25; // temperatuur bij de nominale weerstand van de ntc
const int beta = 3435; // Beta van de ntc

float weerstand_ntc(int meetwaarde) {
  return weerstand_in_serie * ((1023 / (float)meetwaarde) - 1);
}

float temperatuur_ntc(float weerstand) {
  float temperatuur;
  temperatuur = weerstand / nominale_weerstand; // (R/Ro)
  temperatuur = log(temperatuur); // ln(R/Ro)
  temperatuur /= beta; // 1/B * ln(R/Ro)
  temperatuur += 1.0 / (nominale_temperatuur + 273.15); // + (1/To)
  temperatuur = 1.0 / temperatuur; // Inverteer
  return temperatuur - 273.15; // Converteer van Kelvin naar Celsius
}

void verbind_wifi() {
  
  // Verbind met WiFi-netwerk
  Serial.println();
  Serial.println();
  Serial.print("Verbind met ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);
 
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi verbonden");
  Serial.print("IP-adres: ");
  Serial.println(WiFi.localIP());  
}  

void zend_waarde(float temperatuur) {
  
  Serial.print("Verbind met ");
  Serial.println(domoticz_server);
  
  WiFiClient client;
  if (!client.connect(domoticz_server, domoticz_poort)) {
    Serial.println("Verbinding mislukt");
    client.stop();
    return;
  }

  Serial.println("Vraag URL op");

  client.print("GET /json.htm?type=command&param=udevice&idx=");
  client.print(idx);
  client.print("&nvalue=0&svalue=");
  client.print(temperatuur);
  client.println(" HTTP/1.1");
  client.print("Host: ");
  client.print(domoticz_server);
  client.print(":");
  client.println(domoticz_poort);
  client.println("User-Agent: ESP8266");
  client.println("Connection: close");
  client.println();
        
  client.stop();
}

void setup() {
  Serial.begin(115200);

  uint32_t teller;
  uint32_t meetwaarde;

  pinMode(puls, OUTPUT);
  digitalWrite(puls, LOW);
  
  system_rtc_mem_read(TELLER_MEM, &teller, 4);
  system_rtc_mem_read(MEETWAARDE_MEM, &meetwaarde, 4);

  if (teller >> 8 != MAGIC) {
     Serial.println("Eerste keer");
     teller = MAGIC << 8; // Laagste byte van de teller staat nu op 0
     system_rtc_mem_write(TELLER_MEM, &teller, 1);
     meetwaarde = 0;
     system_rtc_mem_write(MEETWAARDE_MEM, &meetwaarde, 4);
     ESP.reset();    
  } else {
    if (teller%256 != samples) {
       teller += 1;
       Serial.print("teller = ");
       Serial.println(teller%256);
       system_rtc_mem_write(TELLER_MEM, &teller, 1);
     
       digitalWrite(puls, HIGH);
       meetwaarde += analogRead(sensor);
       digitalWrite(puls, LOW);
       Serial.print("meetwaarde = ");
       Serial.println(meetwaarde);
       system_rtc_mem_write(MEETWAARDE_MEM, &meetwaarde, 4);

       ESP.deepSleep(SECONDE); // Ga 1 seconde in deep sleep tot volgende meting
     
    } else {
      teller = (MAGIC - 1) << 8;
      system_rtc_mem_write(TELLER_MEM, &teller, 1);  
        
      int meting = meetwaarde / samples;
  
      Serial.println();
      Serial.print("Meting analoge pin: ");
      Serial.println(meting);

      float weerstand = weerstand_ntc(meting);
      Serial.print("Weerstand ntc: ");
      Serial.print(weerstand);
      Serial.println(" Ohm");

      float temperatuur = round(temperatuur_ntc(weerstand)*10)/10.0;
      Serial.print("Temperatuur: ");
      Serial.print(temperatuur);
      Serial.println(" °C");

      verbind_wifi();
      zend_waarde(temperatuur);
      Serial.println("Naar deep sleep");
      ESP.deepSleep(slapen);
    }
  }

}
 
void loop() {  
}
