// Project 2: Weeralarm

#include <ESP8266WiFi.h>

const int rood = 5; // D1
const int oranje = 4; // D2
const int geel = 14; // D5
const int groen = 12; // D6
const int wachttijd = 300000; // 5 minuten

const char* ssid = "SSID";
const char* password = "WACHTWOORD";
const char* host = "reshift.nl";
const int httpPort = 80;
const String url = "/weeralarm.php?regio=utrecht"; // Vervang eventueel utrecht door de gewenste provincie (gebruik kleine letters).
const String code_rood="warning-overview--red";
const String code_oranje="warning-overview--orange";
const String code_geel="warning-overview--yellow";
const String code_groen="warning-overview--green";

void setup() {
  Serial.begin(115200);

  pinMode(rood, OUTPUT);
  pinMode(oranje, OUTPUT);
  pinMode(geel, OUTPUT);
  pinMode(groen, OUTPUT);
  digitalWrite(rood, HIGH);
  digitalWrite(oranje, HIGH);
  digitalWrite(geel, HIGH);
  digitalWrite(groen, HIGH);

  // Verbind met WiFi-netwerk
  Serial.println();
  Serial.println();
  Serial.print("Verbind met ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi verbonden");
  Serial.print("IP-adres: ");
  Serial.println(WiFi.localIP());
}

void loop() {

  int gevonden = 0;
  
  digitalWrite(rood, LOW);
  digitalWrite(oranje, LOW);
  digitalWrite(geel, LOW);
  digitalWrite(groen, LOW);

  Serial.print("Verbind met ");
  Serial.println(host);
  
  WiFiClient client;
  if (!client.connect(host, httpPort)) {
    Serial.println("Verbinding mislukt");
    client.stop();
    return;
  }

  Serial.print("Vraag URL op: ");
  Serial.println(url);

  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
                 "Host: " + host + "\r\n\r\n");
               
  unsigned long timeout = millis();
  while (client.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial.println("Verbinding mislukt: timeout");
      client.stop();
      return;
    }
  }

  while(client.available() && gevonden==0){
    String regel = client.readStringUntil('\r');
    if(regel.indexOf(code_rood) != -1) {
      digitalWrite(rood, HIGH);
      gevonden = 1;
    } else if(regel.indexOf(code_oranje) != -1) {
      digitalWrite(oranje, HIGH);
      gevonden = 1;
    } else if(regel.indexOf(code_geel) != -1) {
      digitalWrite(geel, HIGH);
      gevonden = 1;
    } else if(regel.indexOf(code_groen) != -1) {
      digitalWrite(groen, HIGH);
      gevonden = 1;
    } 
  }

  Serial.println();
  Serial.println("Verbreek verbinding");
  client.stop();
  if(gevonden==1){
    delay(wachttijd);
  }
}
