// Project 3: Weermonitor
#include <ESP8266WiFi.h>
// Wire.h is nodig voor de I2C-communicatie van het OLED-scherm.
#include <Wire.h>
#include "OLED.h"

// Definieer het OLED-scherm met display(SDA, SCL);
// Op SDA van het scherm sluiten we pin D2 (GPIO4) van de ESP8266 aan.
// Op SCL van het scherm sluiten we pin D1 (GPIO5) van de RSP8266 aan.
OLED display(4, 5); // D2, D1

const int wachttijd = 300000; // 5 minuten

const char* ssid = "SSID";
const char* password = "WACHTWOORD";

const char* host = "reshift.nl";
const int httpPort = 80;
const String url="/weermonitor.php?weerstation=6260";
const String weerstation="6260"; // Meetstation De Bilt

WiFiClient client;

void toon_boodschap(String boodschap, int rij = 0, int kolom = 0) {
  char boodschap_char[boodschap.length()+1];
  boodschap.toCharArray(boodschap_char, boodschap.length()+1);
  display.print(boodschap_char, rij, kolom);
}

void lees_tot_tag(String begin_tag, String einde_tag, String toevoeging = "", int rij = 0, int kolom = 0) {
  String inhoud = "";
  while(client.available() && inhoud==""){
    String regel = client.readStringUntil('\r');
    int begin_pos = regel.indexOf(begin_tag);
    if(begin_pos != -1) {
      int einde_pos = regel.indexOf(einde_tag);
      inhoud = regel.substring(begin_pos + begin_tag.length(), einde_pos) + toevoeging;
      toon_boodschap(inhoud, rij, kolom);
    }
  }
}

void setup() {
 Serial.begin(115200);

  // Verbind met WiFi-netwerk
  Serial.println();
  Serial.println();
  Serial.print("Verbind met ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi verbonden");
  Serial.print("IP-adres: ");
  Serial.println(WiFi.localIP());
}

void loop() {

  int weerstation_gevonden = 0;

  display.begin();
  
  Serial.print("Verbind met ");
  Serial.println(host);
  
  if (!client.connect(host, httpPort)) {
    Serial.println("Verbinding mislukt");
    client.stop();
    return;
  }

  Serial.print("Vraag URL op: ");
  Serial.println(url);

  client.print(String("GET ") + url + " HTTP/1.1\r\n" +
                 "Host: " + host + "\r\n\r\n");
               
  unsigned long timeout = millis();
  while (client.available() == 0) {
    if (millis() - timeout > 5000) {
      Serial.println("Verbinding mislukt: timeout");
      client.stop();
      return;
    }
  }

  while(client.available() && weerstation_gevonden==0){
    String regel = client.readStringUntil('\r');
    if(regel.indexOf("<stationcode>" + weerstation) != -1)
      weerstation_gevonden = 1;
  }

  if(weerstation_gevonden==0) {
    toon_boodschap("Weerstation " + weerstation + " niet gevonden");
  } else {
    lees_tot_tag("<luchtvochtigheid>", "</luchtvochtigheid>", "% vochtigheid");
    lees_tot_tag("<temperatuurGC>", "</temperatuurGC>", " graden", 1);
    lees_tot_tag("<windsnelheidBF>", "</windsnelheidBF>", " Bft", 2, 4);
    lees_tot_tag("<windrichting>", "</windrichting>", "", 2);
    lees_tot_tag(" zin=\"", "\">", "", 3);
  }

  Serial.println();
  Serial.println("Verbreek verbinding");
  client.stop();
  delay(wachttijd);
}

