// Project 4: Berichten
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>
// Wire.h is nodig voor de I2C-communicatie van het OLED-scherm.
#include <Wire.h>
#include "OLED.h"

const char* ssid = "SSID";
const char* password = "WACHTWOORD";

// Definieer het OLED-scherm met display(SDA, SCL);
// Op SDA van het scherm sluiten we pin D2 (GPIO4) van de ESP8266 aan.
// Op SCL van het scherm sluiten we pin D1 (GPIO5) van de RSP8266 aan.
OLED display(4, 5); // D2, D1

ESP8266WebServer server(80);

// Configureer een statisch ip-adres. Pas deze gegevens aan je thuisnetwerk aan.
IPAddress ip(192, 168, 0, 201); 
IPAddress gateway(192, 168, 0, 1);
IPAddress subnet(255, 255, 255, 0);

const char* www_gebruikersnaam = "admin"; // Veranderen!
const char* www_wachtwoord = "esp8266"; // Veranderen!

void toon_boodschap(String boodschap, int rij = 0, int kolom = 0) {
  char boodschap_char[boodschap.length()+1];
  boodschap.toCharArray(boodschap_char, boodschap.length()+1);
  display.print(boodschap_char, rij, kolom);
}

void handleRoot() {

  Serial.println("Client verbonden");

  if(!server.authenticate(www_gebruikersnaam, www_wachtwoord))
    server.requestAuthentication();

  // Stuur een antwoord terug.
  String pagina = "<!DOCTYPE HTML>\n";
  pagina += "<html><head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\"></head><div style=\"text-align:center\"><h1>Stuur een bericht</h1>\n";
  pagina += "<form action=\"/submit\" method=\"POST\">\n";
  pagina += "naam: <input type=\"text\" name=\"naam\"><br>\n";
  pagina += "bericht: <input type=\"text\" name=\"bericht\"><br>\n";
  pagina += "<input type=\"submit\" value=\"Zend bericht\">\n";
  pagina += "</form>\n";
  pagina += "</div></html>\n";
  server.send(200, "text/html", pagina);
}

void handleSubmit() {

  if(!server.authenticate(www_gebruikersnaam, www_wachtwoord))
    server.requestAuthentication();  
    
  display.begin();
  toon_boodschap("<" + server.arg("naam") + ">");
  toon_boodschap(server.arg("bericht"), 1);
  server.send(200, "text/plain", "OK");

}

void setup() {
  Serial.begin(115200);
 
  // Verbind met WiFi-netwerk
  Serial.println();
  Serial.println();
  Serial.print("Verbind met ");
  Serial.println(ssid);

  WiFi.config(ip, gateway, subnet);
  WiFi.begin(ssid, password);
 
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi verbonden");

  server.on("/", handleRoot);
  server.on("/submit", handleSubmit);
 
  // Start de server
  server.begin();
  Serial.println("Server opgestart");
 
  // Toon het ip-adres
  Serial.print("Gebruik deze url om te verbinden: ");
  Serial.print("http://");
  Serial.print(WiFi.localIP());
  Serial.println("/");
 
}
 
void loop() {
  server.handleClient();
}
