// Project 5: Wake-uplight 

#include <ESP8266WiFi.h>
#include <WiFiUdp.h>

const int ledPin = 4; // GPIO4 = D2
const int ledHelderheden[] =  {0, 1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1023}; 
const int ledInterval = 30000; // Verhoog helderheid elke 30 seconden
const int aantalHelderheden = sizeof(ledHelderheden) / sizeof(int);

// Laat het alarm afgaan om 07:30
const int alarmUur = 7;
const int alarmMinuut = 30;

const char* ssid = "SSID";
const char* password = "WACHTWOORD";

const int wachttijd = 300000; // Vraag tijd op elke 5 minuten

const char* ntpservername = "0.nl.pool.ntp.org";
IPAddress ntpserverIP;

const int ntpPacketSize = 48;
byte ntpBuffer[ntpPacketSize];
WiFiUDP UDP;
const int poort = 123;

unsigned long lastNTPResponse = millis();
unsigned long prevNTP = 0;
uint32_t timeUNIX = 0;
unsigned long prevActualTime = 0;

const int dstVerschil = 2; // Kies 1 voor wintertijd en 2 voor zomertijd. Dit is het aantal uren verschil met UTC.

uint32_t getTime() {
  if (UDP.parsePacket() == 0) { // Er is nog geen antwoord
    return 0;
  }
  UDP.read(ntpBuffer, ntpPacketSize); // Lees het pakket in en sla het op in de buffer
  // Combineer de 4 bytes van de timestamp in één 32-bit getal
  uint32_t ntpTime = (ntpBuffer[40] << 24) | (ntpBuffer[41] << 16) | (ntpBuffer[42] << 8) | ntpBuffer[43];
  // UNIX-timestamps beginnen op 1 januari 1970
  // In NTP-tijd is dat 70 jaar of 2208988800 seconden.
  // Om NTP-tijd naar UNIX-tijd om te zetten, trekken we dus 70 jaar van NTP-tijd af.
  return ntpTime - 2208988800UL;
}

void sendNTPpacket(IPAddress& adres) {
  memset(ntpBuffer, 0, ntpPacketSize); // Zet alle bytes in de buffer op 0
  // Initialiseer de waardes voor een NTP-aanvraag
  ntpBuffer[0] = 0b11100011; // LI, Version, Mode
  // Zend een pakket om een timestamp te vragen:
  UDP.beginPacket(adres, poort);
  UDP.write(ntpBuffer, ntpPacketSize);
  UDP.endPacket();
}

inline int getSeconds(uint32_t UNIXTime) {
  return UNIXTime % 60;
}

inline int getMinutes(uint32_t UNIXTime) {
  return UNIXTime / 60 % 60;
}

inline int getHours(uint32_t UNIXTime) {
  return UNIXTime / 3600 % 24;
}

void setup() {
  Serial.begin(115200);

  pinMode(ledPin, OUTPUT);
  analogWrite(ledPin, ledHelderheden[0]);

  // Verbind met WiFi-netwerk
  Serial.println();
  Serial.println();
  Serial.print("Verbind met ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi verbonden");
  Serial.print("IP-adres: ");
  Serial.println(WiFi.localIP());

  Serial.println("Start UDP-verbinding");
  UDP.begin(poort);
  Serial.print("Luister naar UDP-pakketten op lokale poort ");
  Serial.println(UDP.localPort());
  Serial.println();

  if(!WiFi.hostByName(ntpservername, ntpserverIP)) { // Vraag het ip-adres van de ntp-server op
    Serial.println("DNS-aanvraag mislukt. Reboot...");
    Serial.flush();
    ESP.reset();
  }
  Serial.print("IP-adres ntp-server: ");
  Serial.println(ntpserverIP);
  
  Serial.println("NTP-aanvraag verzenden...");
  sendNTPpacket(ntpserverIP);  
}

void loop() {
  unsigned long currentMillis = millis();

  if (currentMillis - prevNTP > wachttijd) {
    prevNTP = currentMillis;
    Serial.println("NTP-aanvraag verzenden...");
    sendNTPpacket(ntpserverIP);
  }

  uint32_t time = getTime();
  if (time) { // Heeft de ntp-server al een nieuwe tijd teruggestuurd?
    timeUNIX = time;
    Serial.print("NTP-antwoord: ");
    Serial.println(timeUNIX);
    lastNTPResponse = currentMillis;
  } else if ((currentMillis - lastNTPResponse) > 3600000) {
    Serial.println("Meer dan een uur sinds het laatste antwoord van de NTP-server. Reboot...");
    Serial.flush();
    ESP.reset();
  }

  uint32_t actualTime = timeUNIX + (currentMillis - lastNTPResponse)/1000 + 3600 * dstVerschil;
  if (actualTime != prevActualTime && timeUNIX != 0) { // Als er een seconde sinds de laatste keer is verstreken
    prevActualTime = actualTime;
    Serial.printf("Lokale tijd: %d:%d:%d\n", getHours(actualTime), getMinutes(actualTime), getSeconds(actualTime));
  }  

  if(getHours(actualTime) == alarmUur && getMinutes(actualTime) == alarmMinuut && getSeconds(actualTime) == 00) {
    Serial.printf("Alarm: %d:%d:%d\n", getHours(actualTime), getMinutes(actualTime), getSeconds(actualTime));
    for(int i = 0; i < aantalHelderheden; i++) {
      Serial.printf("Helderheid %d\n", i);
      analogWrite(ledPin, ledHelderheden[i]);
      delay(ledInterval);
    }
  }

}
