// Project 8: Domoticasensor 
#include <ESP8266WiFi.h>

const char* ssid = "SSID";
const char* password = "WACHTWOORD";
const int puls = 5; // D1
const int sensor = 0; // A0
const int samples = 300; // Aantal samples. Tevens het aantal seconden tussen twee metingen.

const char* domoticz_server = "192.168.0.101"; // Wijzig naar het ip-adres van je Domoticz-server.
const int domoticz_poort = 8080;
const int idx = 1; // idx van de virtuele sensor in Domoticz, te vinden in Instellingen / Apparaten.

const int weerstand_in_serie = 10000; // 10 kilo-ohm
const int nominale_weerstand = 10000; // nominale weerstand van de ntc
const int nominale_temperatuur = 25; // temperatuur bij de nominale weerstand van de ntc
const int beta = 3435; // Beta van de ntc

float weerstand_ntc(int meetwaarde) {
  return weerstand_in_serie * ((1023 / (float)meetwaarde) - 1);
}

float temperatuur_ntc(float weerstand) {
  float temperatuur;
  temperatuur = weerstand / nominale_weerstand; // (R/Ro)
  temperatuur = log(temperatuur); // ln(R/Ro)
  temperatuur /= beta; // 1/B * ln(R/Ro)
  temperatuur += 1.0 / (nominale_temperatuur + 273.15); // + (1/To)
  temperatuur = 1.0 / temperatuur; // Inverteer
  return temperatuur - 273.15; // Converteer van Kelvin naar Celsius
}

int sample(int sensor, int keren) {
  int waarde = 0;
  for(int i = 0; i < keren; i++) {
    digitalWrite(puls, HIGH);
    waarde += analogRead(sensor);
    digitalWrite(puls, LOW);
    delay(1000);
  }
  return waarde / keren;
}

void setup() {
  Serial.begin(115200);

  pinMode(puls, OUTPUT);
  digitalWrite(puls, LOW);

  // Verbind met WiFi-netwerk
  Serial.println();
  Serial.println();
  Serial.print("Verbind met ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);
 
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi verbonden");
  Serial.print("IP-adres: ");
  Serial.println(WiFi.localIP());
}
 
void loop() {

  Serial.println("Meetgegevens verzamelen");
  int meting = sample(sensor, samples);
  Serial.println();
  Serial.print("Meting analoge pin: ");
  Serial.println(meting);

  float weerstand = weerstand_ntc(meting);
  Serial.print("Weerstand ntc: ");
  Serial.print(weerstand);
  Serial.println(" Ohm");

  float temperatuur = round(temperatuur_ntc(weerstand)*10)/10.0;
  Serial.print("Temperatuur: ");
  Serial.print(temperatuur);
  Serial.println(" °C");

  Serial.print("Verbind met ");
  Serial.println(domoticz_server);
  
  WiFiClient client;
  if (!client.connect(domoticz_server, domoticz_poort)) {
    Serial.println("Verbinding mislukt");
    client.stop();
    return;
  }

  Serial.println("Vraag URL op");

  client.print("GET /json.htm?type=command&param=udevice&idx=");
  client.print(idx);
  client.print("&nvalue=0&svalue=");
  client.print(temperatuur);
  client.println(" HTTP/1.1");
  client.print("Host: ");
  client.print(domoticz_server);
  client.print(":");
  client.println(domoticz_poort);
  client.println("User-Agent: ESP8266");
  client.println("Connection: close");
  client.println();
        
  client.stop();
  
}
